////////////////////////////////////////////////////////
//
// SourceMod Script
//
// DoD Change Objective State
//
// Developed by FeuerSturm for the Realism Community!
//
////////////////////////////////////////////////////////
#include <sourcemod>
#include <sdktools>
#include <dodhooks>


#define PLUGIN_VERSION "1.3"

#define DISABLE		0
#define ENABLE		1
#define NOCHANGE	2

public Plugin:myinfo = 
{
	name = "DoD Disable Objectives",
	author = "FeuerSturm",
	description = "Disable capping flags/areas/targets on the fly!",
	version = PLUGIN_VERSION,
	url = "http://www.dodsourceplugins.net"
}

new String:Objectives[5][] =
{
	"dod_capture_area", "dod_control_point", "dod_bomb_target", "dod_bomb_dispenser", "dod_bomb_dispenser_icon"
}

new Handle:ObjectiveState = INVALID_HANDLE
new Handle:ObjectiveFullCap = INVALID_HANDLE
new Handle:ObjectiveMinPlayers = INVALID_HANDLE
new String:FlagSnd[] = "ambient/flag.wav"
new bool:g_EmitSound = false
new bool:g_SoundLooping = false

public OnPluginStart()
{
	ObjectiveState = CreateConVar("dod_objectivestate", "2", "<0/1/2> = set Objectives to  -  0 = Disabled  -  1 = Enabled  -  2 = Don't change state (default)", FCVAR_PLUGIN, true, 0.0, true, 2.0)
	ObjectiveFullCap = CreateConVar("dod_objectivefullcap", "0", "<1/0> = allow/disallow FullCap", FCVAR_PLUGIN, true, 0.0, true, 1.0)
	ObjectiveMinPlayers = CreateConVar("dod_objectiveminplayers", "0", "<#> = minimum number of players ingame to enable Objectives (ONLY works with dod_objectivestate 1)", FCVAR_PLUGIN, true, 0.0, true, 32.0)
	HookConVarChange(ObjectiveState, UpdateObjectiveState)
	HookConVarChange(ObjectiveMinPlayers, UpdateObjectiveMinPlayers)
	HookEventEx("dod_round_active", OnRoundActive, EventHookMode_Post)
	HookEventEx("dod_round_start", OnRoundStart, EventHookMode_Post)
	AddAmbientSoundHook(AmbientSHook:HookFlagSound)
}

public OnClientPutInServer(client)
{
	CheckPlayerCount()
}

public OnClientDisconnect_Post(client)
{
	CheckPlayerCount()
}

public OnMapStart()
{
	g_EmitSound = false
	g_SoundLooping = false
	PrecacheSound(FlagSnd, true)
}

public OnMapEnd()
{
	g_SoundLooping = false
	g_EmitSound = false
}

public EmitFlagSound(ObjState)
{
	new Objective = -1
	while((Objective = FindEntityByClassname(Objective, "dod_control_point")) != -1)
	{
		new Float:Origin[3]
		GetEntPropVector(Objective, Prop_Send, "m_vecOrigin", Origin)
		if(GetConVarInt(ObjectiveState) == DISABLE || ObjState == DISABLE)
		{
			if(g_SoundLooping == true)
			{
				g_EmitSound = true
				EmitAmbientSound(FlagSnd, Origin, Objective, SNDLEVEL_NORMAL, SND_STOPLOOPING, 0.4, SNDPITCH_NORMAL, 0.0)
				LogToGame("Stopping LoopSound!")
			}
		}
		else
		{
			if(g_SoundLooping == false)
			{
				g_EmitSound = true
				EmitAmbientSound(FlagSnd, Origin, Objective, SNDLEVEL_NORMAL, SND_NOFLAGS, 0.4, SNDPITCH_NORMAL, 0.0)
				LogToGame("Starting LoopSound!")
			}
		}
	}
	if(GetConVarInt(ObjectiveState) == DISABLE || ObjState == DISABLE)
	{
		if(g_SoundLooping == true)
		{
			g_SoundLooping = false
		}
	}
	else
	{
		if(g_SoundLooping == false)
		{
			g_SoundLooping = true
		}
	}
}

public Action:OnSetWinningTeam(index)
{
	if(GetConVarInt(ObjectiveState) != NOCHANGE && GetConVarInt(ObjectiveFullCap) == 0)
	{
		return Plugin_Handled
	}
	return Plugin_Continue
}

public Action:HookFlagSound(String:sample[PLATFORM_MAX_PATH], &entity, &Float:volume, &level, &pitch, Float:pos[3], &flags, &Float:delay)
{
	if(StrEqual(sample, FlagSnd))
	{
		if(g_EmitSound == true)
		{
			g_EmitSound = false
			return Plugin_Continue
		}
		else
		{
			flags = SND_STOPLOOPING
			return Plugin_Changed
		}
	}
	return Plugin_Continue
}

public CheckPlayerCount()
{
	if(GetConVarInt(ObjectiveState) == ENABLE)
	{
		new playercount = GetClientCount(true)
		if(playercount >= GetConVarInt(ObjectiveMinPlayers))
		{
			SetObjectiveState(ENABLE)
		}
		else
		{
			SetObjectiveState(DISABLE)
		}
	}
}

public SetObjectiveState(ObjState)
{
	if(ObjState != NOCHANGE)
	{
		for(new Obj = 0; Obj < 4; Obj++)
		{
			new Objective = -1
			while((Objective = FindEntityByClassname(Objective, Objectives[Obj])) != -1)
			{
				AcceptEntityInput(Objective, ObjState == ENABLE ? "Enable" : "Disable")
				if(StrEqual(Objectives[Obj], "dod_control_point"))
				{
					SetEntityModel(Objective, ObjState == ENABLE ? "models/mapmodels/flags.mdl" : "")
				}
			}
		}
		for(new CP = 0; CP < 7; CP++)
		{
			SetCPVisible(CP, ObjState == ENABLE ? true : false)
		}
	}
	EmitFlagSound(ObjState)
}	
	
public UpdateObjectiveState(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if(!StrEqual(oldValue, newValue))
	{
		new ObjState = StringToInt(newValue)
		if(ObjState == ENABLE)
		{
			CheckPlayerCount()
		}
		else
		{
			SetObjectiveState(ObjState)
		}
	}
}

public UpdateObjectiveMinPlayers(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if(!StrEqual(oldValue, newValue))
	{
		if(GetConVarInt(ObjectiveState) == ENABLE)
		{
			CheckPlayerCount()
		}
	}
}

public Action:OnRoundStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	EmitFlagSound(DISABLE)
	return Plugin_Continue
}

public Action:OnRoundActive(Handle:event, const String:name[], bool:dontBroadcast)
{
	EmitFlagSound(ENABLE)
	new ObjState = GetConVarInt(ObjectiveState)
	if(ObjState == ENABLE)
	{
		CheckPlayerCount()
		return Plugin_Continue
	}
	SetObjectiveState(ObjState)
	return Plugin_Continue
}